@@ -40,6 +40,19 @@
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif /* HAVE_SYS_SELECT_H */
+ 
+/* if we have a recent enough glibc, use its __abort_msg variable for storing
+ * assertion messages (just like assert()). If not, declare our own variable,
+ * so that platforms with older glibc or different libc implementations can use
+ * this feature for debugging as well.
+ */
+#ifdef HAVE_LIBC_ABORT_MSG
+extern char *__abort_msg;
+#define ASSERT_MESSAGE_STORE __abort_msg
+#else
+char *__glib_assert_msg = NULL;
+#define ASSERT_MESSAGE_STORE __glib_assert_msg
+#endif
 
 /* --- structures --- */
 struct GTestCase
@@ -1297,6 +1310,16 @@ g_assertion_message (const char     *domain,
                    func, func[0] ? ":" : "",
                    " ", message, NULL);
   g_printerr ("**\n%s\n", s);
+
+  /* store assertion message in global variable, so that it can be found in a
+   * core dump; also, use standard C allocation here for compatiblity with
+   * glibc's __abort_msg variable */
+  if (ASSERT_MESSAGE_STORE != NULL)
+      /* free the old one */
+      free (ASSERT_MESSAGE_STORE);
+  ASSERT_MESSAGE_STORE = (char*) malloc (strlen (s) + 1);
+  strcpy (ASSERT_MESSAGE_STORE, s);
+
   g_test_log (G_TEST_LOG_ERROR, s, NULL, 0, NULL);
   g_free (s);
   abort();
