@@ -1087,6 +1087,10 @@ g_object_new (GType	   object_type,
   
   g_return_val_if_fail (G_TYPE_IS_OBJECT (object_type), NULL);
   
+  /* short circuit for calls supplying no properties */
+  if (!first_property_name)
+    return g_object_newv (object_type, 0, NULL);
+
   va_start (var_args, first_property_name);
   object = g_object_new_valist (object_type, first_property_name, var_args);
   va_end (var_args);
@@ -1334,7 +1338,7 @@ g_object_new_valist (GType	  object_type,
 
   class = g_type_class_ref (object_type);
 
-  params = g_new (GParameter, n_alloced_params);
+  params = g_new0 (GParameter, n_alloced_params);
   name = first_property_name;
   while (name)
     {
@@ -1357,9 +1361,8 @@ g_object_new_valist (GType	  object_type,
 	  params = g_renew (GParameter, params, n_alloced_params);
 	}
       params[n_params].name = name;
-      params[n_params].value.g_type = 0;
-      g_value_init (&params[n_params].value, G_PARAM_SPEC_VALUE_TYPE (pspec));
-      G_VALUE_COLLECT (&params[n_params].value, var_args, 0, &error);
+      G_VALUE_COLLECT_INIT (&params[n_params].value, G_PARAM_SPEC_VALUE_TYPE (pspec),
+			    var_args, 0, &error);
       if (error)
 	{
 	  g_warning ("%s: %s", G_STRFUNC, error);
@@ -1472,9 +1475,8 @@ g_object_set_valist (GObject	 *object,
           break;
         }
 
-      g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (pspec));
-      
-      G_VALUE_COLLECT (&value, var_args, 0, &error);
+      G_VALUE_COLLECT_INIT (&value, G_PARAM_SPEC_VALUE_TYPE (pspec), var_args,
+			    0, &error);
       if (error)
 	{
 	  g_warning ("%s: %s", G_STRFUNC, error);
